@@ -43,6 +43,7 @@
 #include "launcher-iconview.h"
 #include "launcher-link-editor.h"
 #include "launcher-menu.h"
+#include "launcher-shortcut.h"
 #include "launcher-sidebar.h"
 #include "launcher-startup.h"
 #include "launcher-util.h"
@@ -73,6 +74,10 @@ struct _LauncherAppPrivate
   ClutterActor *iconviewbg;
   ClutterActor *iconview;
   ClutterActor *sidebar;
+  ClutterActor *places;
+
+  LauncherMenuCategory *current_cat;
+  LauncherMenuCategory *places_cat;
 
   ClutterTimeline       *time;
   ClutterEffectTemplate *temp;
@@ -94,6 +99,11 @@ on_new_category (LauncherCatbar       *catbar,
   g_return_if_fail (category);
   priv = app->priv;
 
+  priv->current_cat = category;
+
+  if (priv->places && priv->current_cat != priv->places_cat)
+    launcher_shortcut_set_active (LAUNCHER_SHORTCUT (priv->places), FALSE);
+
   if (CLUTTER_IS_ACTOR (priv->iconview))
   {
     clutter_effect_fade (priv->temp, priv->iconview, 0,
@@ -105,15 +115,14 @@ on_new_category (LauncherCatbar       *catbar,
   clutter_container_add_actor (CLUTTER_CONTAINER (app),
                                priv->iconview);
   clutter_actor_set_position (priv->iconview, 
-                              (CSW()*0.01)+cfg->bar_width, 
-                              PANEL_HEIGHT*3.3+2);
+                         clutter_actor_get_x (priv->window)+cfg->bar_width-3,
+                              PANEL_HEIGHT*2.3+PADDING);
 
   clutter_actor_show (priv->iconview);
   clutter_focus_manager_insert_group(CLUTTER_FOCUS_MANAGER(priv->focus_manager),
                                      CLUTTER_FOCUS_GROUP (priv->iconview),
                                      1);
 
-
   launcher_iconview_set_category (LAUNCHER_ICONVIEW (priv->iconview), category);
 }
 
@@ -247,7 +256,6 @@ on_link_reponse (GtkWidget *dialog, gint res, LauncherApp *app)
   gtk_widget_destroy (dialog);
 }
 
-
 static void
 on_watch_dir_changed (GFileMonitor      *monitor,
                       GFile             *file,
@@ -380,7 +388,7 @@ load_dist_logo (LauncherApp *app)
     clutter_actor_set_anchor_point_from_gravity (tex, CLUTTER_GRAVITY_CENTER);
     clutter_actor_set_position (tex, 
                                 clutter_actor_get_x (priv->window) + (width/2),
-                                CSH() - (height/2));
+                                CSH() - PADDING*2 - (height/2));
     clutter_actor_set_name (tex, url);
     clutter_actor_set_reactive (tex, TRUE);
     clutter_actor_show (tex);
@@ -396,6 +404,89 @@ load_dist_logo (LauncherApp *app)
   g_object_unref (client);
 }
 
+static gboolean
+deactivate_shortcut (LauncherShortcut *shortcut)
+{
+  launcher_shortcut_set_active (LAUNCHER_SHORTCUT (shortcut), FALSE);
+  return FALSE;
+}
+
+static void
+on_quit_clicked (LauncherShortcut *shortcut)
+{
+  gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
+                                    "gnome-session-save --kill --gui",
+                                    NULL);
+  g_timeout_add_seconds (2, (GSourceFunc)deactivate_shortcut, shortcut);
+}
+
+static void
+on_places_clicked (LauncherShortcut *shortcut, LauncherApp *app)
+{
+  static LauncherMenuCategory *cat = NULL;
+  GList *places, *p;
+
+  if (cat == app->priv->current_cat)
+    return;
+
+  if (cat)
+  {
+    GList *a;
+    for (a = cat->applications; a; a = a->next)
+    {
+      LauncherMenuApplication *app = a->data;
+
+      g_free (app->name);
+      g_free (app->exec);
+      g_slice_free (LauncherMenuApplication, app);
+    }
+    g_list_free (cat->applications);
+    cat->applications = NULL;
+  }
+  else
+  {
+    cat = app->priv->places_cat = g_slice_new0 (LauncherMenuCategory);
+  }
+
+  places = launcher_sidebar_get_places (LAUNCHER_SIDEBAR 
+                                          (launcher_sidebar_get_default ()));
+  for (p = places; p; p = p->next)
+  {
+    LauncherShortcut *shortcut = p->data;
+    LauncherMenuApplication *app;
+    GVolume *volume;
+    
+    if (!LAUNCHER_IS_SHORTCUT (shortcut))
+      continue;
+    
+    app = g_slice_new0 (LauncherMenuApplication);
+
+    app->name = g_strdup (launcher_shortcut_get_label (shortcut));
+    app->pixbuf = launcher_shortcut_get_icon (shortcut);
+    app->exec = g_strdup_printf ("xdg-open %s", 
+                                 clutter_actor_get_name (CLUTTER_ACTOR (shortcut)));
+    app->can_fav = FALSE;
+
+    volume = g_object_get_data (G_OBJECT (shortcut), "volume");
+    if (G_IS_VOLUME (volume))
+      app->volume = volume;
+
+    cat->applications = g_list_append (cat->applications, app);
+  }
+  
+  launcher_catbar_deactivate (LAUNCHER_CATBAR (app->priv->catbar));
+  on_new_category (NULL, cat, app);
+
+  g_list_free (places);
+}
+
+static void
+on_sidebar_shortcuts_changed (LauncherSidebar *sidebar, LauncherApp *app)
+{
+  app->priv->current_cat = NULL;
+  on_places_clicked (LAUNCHER_SHORTCUT (app->priv->places), app);
+}
+
 /* GObject stuff */
 static void
 launcher_app_finalize (GObject *object)
@@ -426,12 +517,15 @@ launcher_app_init (LauncherApp *app)
   LauncherAppPrivate *priv;
   ClutterActor *stage = clutter_stage_get_default ();
   LauncherConfig *cfg = launcher_config_get_default ();
+  ClutterActor *shortcut;
   const gchar *winmode;
 	
   priv = app->priv = LAUNCHER_APP_GET_PRIVATE (app);
 
   winmode = g_getenv ("LAUNCHER_WINDOWED");
-  priv->windowed = winmode ? atoi (winmode) : FALSE; 
+  priv->windowed = winmode ? atoi (winmode) : FALSE;
+  priv->places = NULL;
+  priv->places_cat = NULL;
   
   /* The desktop background */
   priv->bg = launcher_background_new ();
@@ -460,11 +554,16 @@ launcher_app_init (LauncherApp *app)
   */
 
   /* Window */
-  priv->window = launcher_util_texture_new_from_file (PKGDATADIR"/window.svg");
+  priv->window = launcher_util_texture_new_from_file 
+                                                  (PKGDATADIR"/window.svg");
+  clutter_container_add_actor (CLUTTER_CONTAINER (stage), priv->window);
+  priv->window = tidy_texture_frame_new (CLUTTER_TEXTURE (priv->window),
+                                             30, 30, 30, 30);
   clutter_container_add_actor (CLUTTER_CONTAINER (stage), priv->window);
   clutter_actor_set_size (priv->window, cfg->win_width, cfg->win_height);
   clutter_actor_set_position (priv->window, 
-                              (CSW()-cfg->win_width)/2, PANEL_HEIGHT*2);
+                              (CSW()-cfg->win_width)/2, 
+                              PANEL_HEIGHT + 5);
   clutter_actor_show (priv->window);
 
   /* Ubuntu logo */
@@ -472,7 +571,8 @@ launcher_app_init (LauncherApp *app)
   clutter_container_add_actor (CLUTTER_CONTAINER (stage), priv->ubuntu);
   clutter_actor_set_anchor_point_from_gravity (priv->ubuntu, 
                                                CLUTTER_GRAVITY_NORTH);
-  clutter_actor_set_position (priv->ubuntu, CSW()/2, PANEL_HEIGHT*2.25);
+  clutter_actor_set_position (priv->ubuntu, CSW()/2, 
+                              PANEL_HEIGHT + PADDING + PANEL_HEIGHT/4);
   clutter_actor_show (priv->ubuntu);
 
   /* Init the WM interface */
@@ -491,30 +591,37 @@ launcher_app_init (LauncherApp *app)
                                              30, 30, 30, 30);
   clutter_container_add_actor (CLUTTER_CONTAINER (stage), priv->iconviewbg);
   clutter_actor_set_size (priv->iconviewbg, cfg->iconview_width,
-                          cfg->win_height-(PANEL_HEIGHT*2.3));
+                          cfg->win_height-(PANEL_HEIGHT*1.3+PADDING));
   clutter_actor_set_position (priv->iconviewbg, 
-                              (CSW()*0.01)+cfg->bar_width+2, 
-                              PANEL_HEIGHT*3.3+2);
+                           clutter_actor_get_x (priv->window)+cfg->bar_width-3, 
+                           PANEL_HEIGHT*2.3+PADDING+2);
   clutter_actor_show (priv->iconviewbg);
   clutter_actor_set_reactive (CLUTTER_ACTOR (priv->iconviewbg), TRUE);
   g_signal_connect (priv->iconviewbg, "scroll-event",
                     G_CALLBACK (on_scroll_event), app);
     
   /* Sidebar */
-  priv->sidebar = launcher_sidebar_get_default ();
-  clutter_container_add_actor (CLUTTER_CONTAINER (stage), priv->sidebar);
-  clutter_actor_set_position (priv->sidebar, 
-                              (CSW()*0.01)+CAW(priv->window)-cfg->bar_width,
-                              PANEL_HEIGHT*3.3);
-  clutter_actor_show (priv->sidebar);
-  clutter_focus_manager_insert_group(CLUTTER_FOCUS_MANAGER(priv->focus_manager),
-                                     CLUTTER_FOCUS_GROUP (priv->sidebar),
-                                     1);
+  if (!cfg->is_portrait)
+  {
+    priv->sidebar = launcher_sidebar_get_default ();
+    clutter_container_add_actor (CLUTTER_CONTAINER (stage), priv->sidebar);
+    clutter_actor_set_position (priv->sidebar, 
+                                clutter_actor_get_x (priv->window)
+                                  +CAW(priv->window)-cfg->bar_width,
+                                PANEL_HEIGHT*2.3+PADDING);
+    clutter_actor_show (priv->sidebar);
+    clutter_focus_manager_insert_group 
+                                    (CLUTTER_FOCUS_MANAGER(priv->focus_manager),
+                                       CLUTTER_FOCUS_GROUP (priv->sidebar),
+                                       1);
+  }
+
   /* Category bar */
   priv->catbar = launcher_catbar_get_default ();
   clutter_container_add_actor (CLUTTER_CONTAINER (stage), priv->catbar);
-  clutter_actor_set_position (priv->catbar, (CSW()*0.01)+CAT_PADDING,
-                              PANEL_HEIGHT*3.3);
+  clutter_actor_set_position (priv->catbar, 
+                              clutter_actor_get_x (priv->window),
+                              PANEL_HEIGHT*2.3+PADDING);
   clutter_actor_show (priv->catbar);
   g_signal_connect (priv->catbar, "category-selected",
                     G_CALLBACK (on_new_category), app);
@@ -522,7 +629,33 @@ launcher_app_init (LauncherApp *app)
                                      CLUTTER_FOCUS_GROUP (priv->catbar),
                                      0);
   /* dist logo */
-  load_dist_logo (app);
+  if (!cfg->is_portrait)
+  {
+    load_dist_logo (app);
+  }
+  else
+  {
+    shortcut = launcher_shortcut_new ("gtk-quit",
+                                      _("Quit..."),
+                                      FALSE);
+    clutter_container_add_actor (CLUTTER_CONTAINER (stage), shortcut);
+    clutter_actor_set_position (shortcut, 0, 
+                               cfg->win_height - (cfg->shortcut_height));
+    clutter_actor_show_all (shortcut);
+    g_signal_connect (shortcut, "clicked", G_CALLBACK (on_quit_clicked), NULL);
+
+    shortcut = priv->places = launcher_shortcut_new ("folder",
+                                                     _("Places"),
+                                                     FALSE);
+    clutter_container_add_actor (CLUTTER_CONTAINER (stage), shortcut);
+    clutter_actor_set_position (shortcut, 0, 
+                               cfg->win_height - (cfg->shortcut_height*2));
+    clutter_actor_show_all (shortcut);
+    g_signal_connect (shortcut, "clicked", G_CALLBACK (on_places_clicked), app);
+
+    g_signal_connect (launcher_sidebar_get_default (), "shortcuts-changed",
+                      G_CALLBACK (on_sidebar_shortcuts_changed), app);
+  }
 
   on_new_category (NULL, 
                    launcher_menu_get_categories (priv->menu)->data,
