@@ -32,6 +32,7 @@
 #include <clutter/clutter.h>
 #include <libgnome/gnome-desktop-item.h>
 
+#include "launcher-config.h"
 #include "launcher-defines.h"
 #include "launcher-menu.h"
 
@@ -54,34 +55,6 @@ struct _LauncherMenuPrivate
   GList     *old_apps;
 };
 
-struct _LauncherMenuCategory
-{
-  GList *applications;
-  GdkPixbuf *pixbuf;
-
-  gchar *name;
-  gchar *comment;
-  gchar *icon;
-};
-
-struct _LauncherMenuApplication
-{
-  LauncherMenuCategory *category;
-  GdkPixbuf *pixbuf;
-  ClutterActor *actor;
-
-  gchar *name;
-  gchar *comment;
-  gchar *icon;
-  gchar *exec;
-  gchar *path;
-
-  WnckWindow *window;
-  gint pid;
-
-  gboolean new;
-};
-
 enum 
 {
   MENU_CHANGED,
@@ -406,7 +379,7 @@ make_category (LauncherMenu *menu, GMenuTreeDirectory *dir)
     for (l = priv->old_cats; l; l = l->next)
     {
       LauncherMenuCategory *cat = l->data;
-      if (strcmp (name, cat->name) == 0)
+      if (name && cat->name && strcmp (name, cat->name) == 0)
       {
         category = cat;
         break;
@@ -456,7 +429,7 @@ make_application (LauncherMenu *menu,
     for (a = priv->old_apps; a; a = a->next)
     {
       application = a->data;
-      if (strcmp (path, application->path) == 0)
+      if (path && application->path && strcmp (path, application->path) == 0)
       {
         app = application;
         break;
@@ -483,6 +456,7 @@ make_application (LauncherMenu *menu,
   app->pid = -1;
   app->window = NULL;
   app->new = priv->refresh;
+  app->can_fav = TRUE;
 
   category->applications = g_list_append (category->applications,
                                           (gpointer)app);
@@ -502,8 +476,7 @@ load_menu_from_directory (LauncherMenu *menu,
                           LauncherMenuCategory *category)
 {
   GSList *list, *l;
-  LauncherMenuCategory *root = NULL;
-
+  
   if (dir == NULL)
     return;
 
@@ -531,25 +504,16 @@ load_menu_from_directory (LauncherMenu *menu,
       case GMENU_TREE_ITEM_ENTRY:
         if (category)
           make_application (menu, GMENU_TREE_ENTRY (item), category);
-        else
-        {
-          if (root == NULL)
-            root = make_category (menu, dir);
-          make_application (menu, GMENU_TREE_ENTRY (item), root);        
-        }
-
-        break;
+       break;
 
       case GMENU_TREE_ITEM_SEPARATOR:
-        gmenu_tree_item_unref (item);
-        goto out;
-        
+       ;
+                
       default:
         break;
     }
     gmenu_tree_item_unref (item);
-  }  
-out:
+  }
 
   g_slist_free (list);
 }
@@ -635,7 +599,7 @@ free_menu (LauncherMenu *menu)
   {
     LauncherMenuCategory *cat = l->data;
     
-    if (strcmp (cat->icon, FAVCATNAME)==0)
+    if (cat->icon && strcmp (cat->icon, FAVCATNAME)==0)
     {
       priv->categories = g_list_insert (priv->categories, cat, 0);
 
@@ -669,84 +633,6 @@ free_menu (LauncherMenu *menu)
   i++;
 }
 
-static gboolean
-list_changed (LauncherMenu *menu)
-{
-  LauncherMenuPrivate *priv;
-  GMenuTreeDirectory *root;
-  
-  g_return_val_if_fail (LAUNCHER_IS_MENU (menu), FALSE);
-  priv = menu->priv;
-
-  g_debug ("menu changed\n");
-
-  priv->old_cats = priv->categories;
-  priv->old_apps = priv->menu_apps;
-  priv->categories = priv->menu_apps = NULL;
-
-  priv->refresh = TRUE;
-
-  root = gmenu_tree_get_root_directory (priv->app_tree);
-  load_menu_from_directory (menu, root, NULL);
-  gmenu_tree_item_unref (root);
-
-  priv->refresh = FALSE;
-
-  free_menu (menu);
-
-  /*
-  root = gmenu_tree_get_root_directory (priv->sys_tree);
-  load_menu_from_directory (menu, root, NULL);
-  gmenu_tree_item_unref (root);
-  */
-  g_signal_emit (menu, _menu_signals[MENU_CHANGED], 0);
-  menu->priv->tag = 0;  
-  
-  return FALSE;
-}
-
-/*
- * FIXME: The tree changing should not be destructive. Find a way to make it
- * work without re-creating the entire menu again.
- */
-static void
-tree_changed (GMenuTree *tree, LauncherMenu *menu)
-{
-  LauncherMenuPrivate *priv;
-  GMenuTreeDirectory *root;
-  
-  g_return_if_fail (LAUNCHER_IS_MENU (menu));
-  priv = menu->priv;
-
-  root = gmenu_tree_get_root_directory (tree);
-  gmenu_tree_item_unref (root);  
-  
-  if (priv->tag)
-  {
-    return;
-  }
-  priv->tag = g_timeout_add (500, (GSourceFunc)list_changed, menu);
-}
-
-static void
-on_theme_changed (GtkIconTheme *theme, LauncherMenu *menu)
-{
-  LauncherMenuPrivate *priv;
-  GMenuTreeDirectory *root;
-  
-  g_return_if_fail (LAUNCHER_IS_MENU (menu));
-  priv = menu->priv;
-
-  root = gmenu_tree_get_root_directory (priv->app_tree);
-  gmenu_tree_item_unref (root);  
-  
-  if (priv->tag)
-  {
-    return;
-  }
-  priv->tag = g_timeout_add (500, (GSourceFunc)list_changed, menu);
-}
-
 static void
 sort_categories (LauncherMenu *menu)
 {
@@ -797,6 +683,93 @@ sort_categories (LauncherMenu *menu)
   sort_categories (menu);
 }
 
+static gboolean
+list_changed (LauncherMenu *menu)
+{
+  LauncherMenuPrivate *priv;
+  GMenuTreeDirectory *root;
+  GList *l, *fav = NULL;
+  
+  g_return_val_if_fail (LAUNCHER_IS_MENU (menu), FALSE);
+  priv = menu->priv;
+
+  g_debug ("Menu changed");
+
+  priv->old_cats = priv->categories;
+  priv->old_apps = priv->menu_apps;
+  priv->categories = priv->menu_apps = NULL;
+
+  priv->refresh = TRUE;
+
+  root = gmenu_tree_get_root_directory (priv->app_tree);
+  load_menu_from_directory (menu, root, NULL);
+  gmenu_tree_item_unref (root);
+
+  root = gmenu_tree_get_root_directory (priv->sys_tree);
+  load_menu_from_directory (menu, root, NULL);
+  gmenu_tree_item_unref (root);
+
+  priv->refresh = FALSE;
+
+  free_menu (menu);
+
+  for (l = priv->categories; l; l= l->next)
+  {
+    LauncherMenuCategory *cat = l->data;
+    if (cat->icon && strstr (cat->icon, FAVCATNAME))
+    {
+      fav = l;
+      break;
+    }
+  }
+  if (fav)
+  {
+    LauncherMenuCategory *favcat = fav->data;
+    priv->categories = g_list_delete_link (priv->categories, fav);
+    priv->categories = g_list_prepend (priv->categories, favcat);
+  }
+
+  g_signal_emit (menu, _menu_signals[MENU_CHANGED], 0);
+  menu->priv->tag = 0;  
+  
+  return FALSE;
+}
+
+/*
+ * FIXME: The tree changing should not be destructive. Find a way to make it
+ * work without re-creating the entire menu again.
+ */
+static void
+tree_changed (GMenuTree *tree, LauncherMenu *menu)
+{
+  LauncherMenuPrivate *priv;
+    
+  g_return_if_fail (LAUNCHER_IS_MENU (menu));
+  priv = menu->priv;
+  
+  if (priv->tag)
+  {
+    return;
+  }
+  priv->tag = g_timeout_add (200, (GSourceFunc)list_changed, menu);
+}
+
+static void
+on_theme_changed (GtkIconTheme *theme, LauncherMenu *menu)
+{
+  LauncherMenuPrivate *priv;
+  
+  g_return_if_fail (LAUNCHER_IS_MENU (menu));
+  priv = menu->priv;
+  
+  if (priv->tag)
+  {
+    return;
+  }
+  priv->tag = g_timeout_add (500, (GSourceFunc)list_changed, menu);
+}
+
+
 /* GObject functions */
 static void
 launcher_menu_dispose (GObject *object)
@@ -853,6 +826,7 @@ launcher_menu_init (LauncherMenu *menu)
   priv->refresh = FALSE;
 
   priv->app_tree = load_menu_from_tree (menu, "applications.menu");
+  priv->sys_tree = load_menu_from_tree (menu, "settings.menu");
 
   sort_categories (menu);
 
